﻿///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 2011 EaseVault Technologies Inc.
//    All Rights Reserved
//
//    This software is part of a licensed software product and may
//    only be used or copied in accordance with the terms of that license.
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.Security.Principal;
using System.IO;

namespace CSharpDemo
{
    public class ConfigSetting
    {      
        ListView listView_FilterRules = null;

        uint filterConnectionThreads = 5;
        uint connectionTimeOut = 30;
        EaseFilter.FilterType filterType = EaseFilter.FilterType.FILE_SYSTEM_MONITOR;
        List<FilterRule> filterList = new List<FilterRule>();
        List<uint> excludedPid = new List<uint>();
        Boolean enableFilterTest = false;
        uint requestRegistration = 2863311530;

        public ConfigSetting(ListView lvFilterRules)
        {
            this.listView_FilterRules = lvFilterRules;

            FilterRule filterRule = new FilterRule();
            filterRule.AccessFlag = (EaseFilter.AccessFlag)524272;
            filterRule.FilterMask = @"c:\test*";
            filterRule.ReparseMask = @"c:\reparse*";
            filterList.Add(filterRule);

            uint currentPid = (uint)System.Diagnostics.Process.GetCurrentProcess().Id;
            excludedPid.Add(currentPid);

            InitListView();
        }

   

        public struct FilterRule
        {
            public EaseFilter.AccessFlag AccessFlag;
            public string FilterMask;
            public string ReparseMask;
        }

        public void InitListView()
        {
            //init ListView control
            listView_FilterRules.Clear();		//clear control
            //create column header for ListView
            listView_FilterRules.Columns.Add("#", 20, System.Windows.Forms.HorizontalAlignment.Left);
            listView_FilterRules.Columns.Add("AccessFlags", 80, System.Windows.Forms.HorizontalAlignment.Left);
            listView_FilterRules.Columns.Add("FilterMask", 150, System.Windows.Forms.HorizontalAlignment.Left);
            listView_FilterRules.Columns.Add("ReparseMask", 150, System.Windows.Forms.HorizontalAlignment.Left);


            foreach (FilterRule rule in filterList)
            {
                AddItem(rule);
            }

        }

        public Boolean EnableFilterTest
        {
            get { return enableFilterTest; }
            set { enableFilterTest = value; }
        }


        public uint FilterConnectionThreads
        {
            get { return filterConnectionThreads; }
            set { filterConnectionThreads = value; }
        }

        public uint RequestRegistration
        {
            get { return requestRegistration; }
            set { requestRegistration = value; }
        }

        public List<uint> ExcludedPid
        {
            get { return excludedPid; }
            set { excludedPid = value; }
        }


        public EaseFilter.FilterType FilterType
        {
            get { return filterType; }
            set { filterType = value; }
        }

        public uint ConnectionTimeOut
        {
            get { return connectionTimeOut; }
            set { connectionTimeOut = value; }
        }

        private void AddItem(FilterRule newRule)
        {
            string[] itemStr = new string[listView_FilterRules.Columns.Count];
            itemStr[0] = listView_FilterRules.Items.Count.ToString();
            itemStr[1] = newRule.AccessFlag.ToString();
            itemStr[2] = newRule.FilterMask;
            itemStr[3] = newRule.ReparseMask;
            ListViewItem item = new ListViewItem(itemStr, 0);
            item.Tag = newRule;
            listView_FilterRules.Items.Add(item);
        }

        public bool AddFilterRule(EaseFilter.AccessFlag accessFlag, string filterMask,string reparseMask)
        {
            foreach (FilterRule rule in filterList)
            {
                if (string.Compare(rule.FilterMask, filterMask) == 0)
                {
                    //the filter already there.
                    return false;
                }
            }

            FilterRule newRule = new FilterRule();
            newRule.AccessFlag = accessFlag;
            newRule.FilterMask = filterMask;
            newRule.ReparseMask = reparseMask;

            filterList.Add(newRule);

            AddItem(newRule); ;

            return true;
        }

        public bool RemoveFilterRule(ListViewItem item)
        {
            listView_FilterRules.Items.Remove(item);
            ConfigSetting.FilterRule rule = (ConfigSetting.FilterRule)item.Tag;
            
            bool ret = filterList.Remove(rule);

            InitListView();

            return ret;

            
        }

        public List<FilterRule> FilterList
        {
            get { return filterList; }
        }
    }
}
