﻿///////////////////////////////////////////////////////////////////////////////
//
//    (C) Copyright 2011 EaseVault Technologies Inc.
//    All Rights Reserved
//
//    This software is part of a licensed software product and may
//    only be used or copied in accordance with the terms of that license.
//
///////////////////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using System.IO;

namespace CSharpDemo
{
    public partial class ReplyDataForm : Form
    {
        EaseFilter.MessageSendData messageSend;
        EaseFilter.MessageReplyData messageReply;
        int listItemNumber = 0;
        bool processReplyDataNeeded = true;

        enum DataBufferStatus
        {
            DisplayData = 0,
            ReturnData,
        }

        public ReplyDataForm(EaseFilter.MessageSendData messageSend, EaseFilter.MessageReplyData returnMessage)
        {
            try
            {
                this.messageSend = messageSend;
                this.messageReply = returnMessage;

                InitializeComponent();

                InitReplyDataFrom();

                if (!ProcessDataBuffer(DataBufferStatus.DisplayData))
                {
                    processReplyDataNeeded = false;
                    return;
                }
            }
            catch (Exception ex)
            {
                EventMessage.LogErrorMessage("Initialize replyDataForm failed:" + ex.Message);
            }
        }

        public bool ProcessReplyDataNeeded
        {
            get { return processReplyDataNeeded; }
        }

        public EaseFilter.MessageReplyData MessageReply
        {
            get { return messageReply; }
        }

        public void InitReplyDataFrom()
        {
            try
            {
                InitListView();
                this.Text = ((EaseFilter.MessageType)messageSend.MessageType).ToString();
                this.label_FileName.Text = "FileName: " + messageSend.FileName;

                if (messageSend.InfoClass != 0)
                {
                    groupBox_ReturnData.Text = ((WinData.FileInfomationClass)messageSend.InfoClass).ToString();
                }
                else if (messageSend.DataBufferLength > 0)
                {
                    groupBox_ReturnData.Text = "Databuffer Length:" + messageSend.DataBufferLength.ToString();
                }
            }
            catch (Exception ex)
            {
                EventMessage.LogErrorMessage("InitReplyDataFrom failed:" + ex.Message);
            }

        }

        public void InitListView()
        {
            //init ListView control
            listView_ReturnData.Clear();		//clear control
            //create column header for ListView
            listView_ReturnData.Columns.Add("#", 40, System.Windows.Forms.HorizontalAlignment.Left);
            listView_ReturnData.Columns.Add("Name", 150, System.Windows.Forms.HorizontalAlignment.Left);
            listView_ReturnData.Columns.Add("Value ( Double click to change the value)", 300, System.Windows.Forms.HorizontalAlignment.Left);

        }

        void AddItem(string itemName, string value)
        {
            string[] listData = new string[listView_ReturnData.Columns.Count];

            listData[0] = listItemNumber.ToString();
            listItemNumber++;
            listData[1] = itemName;
            listData[2] = value;

            ListViewItem lvItem = new ListViewItem(listData, 0);
            listView_ReturnData.Items.Add(lvItem);

        }

        bool ProcessDataBuffer(DataBufferStatus dataStatus)
        {
            bool ret = false;
            try
            {
                EaseFilter.MessageType messageType = (EaseFilter.MessageType)messageSend.MessageType;
                WinData.FileInfomationClass infoClass = (WinData.FileInfomationClass)messageSend.InfoClass;
                uint dataLength = messageSend.DataBufferLength;
                byte[] data = messageSend.DataBuffer;

                listView_ReturnData.Visible = true;
                textBox_Data.Visible = false;

                switch (messageType)
                {
                    case EaseFilter.MessageType.PRE_CREATE:
                        {
                            ret = true;
                            listView_ReturnData.Visible = false;
                            textBox_Data.Visible = true;

                            if (dataStatus == DataBufferStatus.DisplayData)
                            {
                                textBox_Data.Text = messageSend.FileName;
                            }
                            else
                            {
                                string newFileName = textBox_Data.Text;

                                if (!newFileName.StartsWith("\\??\\"))
                                {
                                    newFileName = "\\??\\" + newFileName;
                                }

                                byte[] returnData = Encoding.Unicode.GetBytes(newFileName);
                                Array.Copy(returnData, messageReply.DataBuffer, returnData.Length);
                                messageReply.DataBufferLength =(uint) returnData.Length;

                            }

                            break;
                        }


                    case EaseFilter.MessageType.PRE_CACHE_READ:
                    case EaseFilter.MessageType.POST_CACHE_READ:
                    case EaseFilter.MessageType.PRE_NOCACHE_READ:
                    case EaseFilter.MessageType.POST_NOCACHE_READ:
                    case EaseFilter.MessageType.PRE_PAGING_IO_READ:
                    case EaseFilter.MessageType.POST_PAGING_IO_READ:
                    case EaseFilter.MessageType.PRE_CACHE_WRITE:
                    case EaseFilter.MessageType.POST_CACHE_WRITE:
                    case EaseFilter.MessageType.PRE_NOCACHE_WRITE:
                    case EaseFilter.MessageType.POST_NOCACHE_WRITE:
                    case EaseFilter.MessageType.PRE_PAGING_IO_WRITE:
                    case EaseFilter.MessageType.POST_PAGING_IO_WRITE:
                        {
                            ret = true;
                            listView_ReturnData.Visible = false;
                            textBox_Data.Visible = true;

                            if (dataStatus == DataBufferStatus.DisplayData)
                            {
                                textBox_Data.Text = Encoding.ASCII.GetString(messageSend.DataBuffer);
                            }
                            else
                            {

                                byte[] returnData = Encoding.ASCII.GetBytes(textBox_Data.Text);
                                Array.Copy(returnData, messageReply.DataBuffer, returnData.Length);
                                messageReply.DataBufferLength = (uint)returnData.Length;

                            }

                            break;
                        }
                    case EaseFilter.MessageType.PRE_SET_INFORMATION:
                    case EaseFilter.MessageType.POST_SET_INFORMATION:
                    case EaseFilter.MessageType.PRE_QUERY_INFORMATION:
                    case EaseFilter.MessageType.POST_QUERY_INFORMATION:
                        {
                            ret = true;
                            switch (infoClass)
                            {
                                case WinData.FileInfomationClass.FileRenameInformation:
                                    {
                                        if (dataStatus == DataBufferStatus.DisplayData)
                                        {
                                            //destination name
                                            AddItem("Rename file to new file name", Encoding.Unicode.GetString(data));
                                        }

                                        break;
                                    }
                                case WinData.FileInfomationClass.FileDispositionInformation:
                                    {
                                        if (dataStatus == DataBufferStatus.DisplayData)
                                        {
                                            AddItem("Deleting file" + messageSend.FileName, "");
                                        }
                                        break;
                                    }
                                case WinData.FileInfomationClass.FileEndOfFileInformation:
                                    {
                                        long newFileSize = BitConverter.ToInt64(data, 0);
                                        if (dataStatus == DataBufferStatus.DisplayData)
                                        {
                                            AddItem("Change file size", newFileSize.ToString());
                                        }
                                        else
                                        {
                                            newFileSize = long.Parse(listView_ReturnData.Items[0].SubItems[2].Text);
                                            byte[] returnData = BitConverter.GetBytes(newFileSize);

                                            messageReply.DataBufferLength = (uint)returnData.Length;
                                            Array.Copy(returnData, messageReply.DataBuffer, returnData.Length);

                                        }
                                        break;
                                    }
                                case WinData.FileInfomationClass.FileBasicInformation:
                                    {
                                        WinData.FileBasicInformation basiInfo = new WinData.FileBasicInformation();
                                        GCHandle pinnedPacket = GCHandle.Alloc(data, GCHandleType.Pinned);
                                        basiInfo = (WinData.FileBasicInformation)Marshal.PtrToStructure(
                                            pinnedPacket.AddrOfPinnedObject(), typeof(WinData.FileBasicInformation));
                                        pinnedPacket.Free();

                                        if (dataStatus == DataBufferStatus.DisplayData)
                                        {
                                            AddItem("CreationTime", DateTime.FromFileTime(basiInfo.CreationTime).ToString("dd-MM-yyyy HH:mm:ss"));
                                            AddItem("LastAccessTime", DateTime.FromFileTime(basiInfo.LastAccessTime).ToString("dd-MM-yyyy HH:mm:ss"));
                                            AddItem("LastWriteTime", DateTime.FromFileTime(basiInfo.LastWriteTime).ToString("dd-MM-yyyy HH:mm:ss"));
                                            AddItem("ChangeTime", DateTime.FromFileTime(basiInfo.ChangeTime).ToString("dd-MM-yyyy HH:mm:ss"));
                                            AddItem("FileAttributes", basiInfo.FileAttributes.ToString());
                                        }
                                        else
                                        {
                                            string creationTime = listView_ReturnData.Items[0].SubItems[2].Text;
                                            string lastAccessTime = listView_ReturnData.Items[1].SubItems[2].Text;
                                            string lastWriteTime = listView_ReturnData.Items[2].SubItems[2].Text;
                                            string changeTime = listView_ReturnData.Items[3].SubItems[2].Text;
                                            string fileAttribute = listView_ReturnData.Items[4].SubItems[2].Text;

                                            basiInfo.CreationTime = DateTime.ParseExact(creationTime, "dd-MM-yyyy HH:mm:ss", System.Globalization.CultureInfo.CurrentCulture).ToFileTime();
                                            basiInfo.LastAccessTime = DateTime.ParseExact(lastAccessTime, "dd-MM-yyyy HH:mm:ss", System.Globalization.CultureInfo.CurrentCulture).ToFileTime();
                                            basiInfo.LastWriteTime = DateTime.ParseExact(lastWriteTime, "dd-MM-yyyy HH:mm:ss", System.Globalization.CultureInfo.CurrentCulture).ToFileTime();
                                            basiInfo.ChangeTime = DateTime.ParseExact(changeTime, "dd-MM-yyyy HH:mm:ss", System.Globalization.CultureInfo.CurrentCulture).ToFileTime();
                                            basiInfo.FileAttributes = uint.Parse(fileAttribute);

                                            uint returnLength = (uint)Marshal.SizeOf(basiInfo);
                                            IntPtr returnData = Marshal.AllocHGlobal((int)returnLength);
                                            Marshal.StructureToPtr(basiInfo, returnData, true);
                                            Marshal.Copy(returnData, messageReply.DataBuffer, 0, (int)returnLength);
                                            messageReply.DataBufferLength = returnLength;
                                            Marshal.FreeHGlobal(returnData);
                                        }

                                        break;
                                    }

                                case WinData.FileInfomationClass.FileStandardInformation:
                                    {
                                        WinData.FileStandardInformation standardInfo = new WinData.FileStandardInformation();
                                        GCHandle pinnedPacket = GCHandle.Alloc(data, GCHandleType.Pinned);
                                        standardInfo = (WinData.FileStandardInformation)Marshal.PtrToStructure(
                                            pinnedPacket.AddrOfPinnedObject(), typeof(WinData.FileStandardInformation));
                                        pinnedPacket.Free();

                                        if (dataStatus == DataBufferStatus.DisplayData)
                                        {
                                            AddItem("FileSize", standardInfo.EndOfFile.ToString());
                                            AddItem("AllocationSize", standardInfo.AllocationSize.ToString());
                                            AddItem("IsDirectory", standardInfo.Directory.ToString());
                                        }
                                        else
                                        {
                                            string FileSize = listView_ReturnData.Items[0].SubItems[2].Text;
                                            string AllocationSize = listView_ReturnData.Items[1].SubItems[2].Text;

                                            standardInfo.EndOfFile = long.Parse(FileSize);
                                            standardInfo.AllocationSize = long.Parse(AllocationSize);

                                            uint returnLength = (uint)Marshal.SizeOf(standardInfo);
                                            IntPtr returnData = Marshal.AllocHGlobal((int)returnLength);
                                            Marshal.StructureToPtr(standardInfo, returnData, true);
                                            Marshal.Copy(returnData, messageReply.DataBuffer, 0, (int)returnLength);
                                            messageReply.DataBufferLength = returnLength;

                                            Marshal.FreeHGlobal(returnData);
                                        }

                                        break;
                                    }
                                case WinData.FileInfomationClass.FileNetworkOpenInformation:
                                    {
                                        WinData.FileNetworkInformation networkInfo = new WinData.FileNetworkInformation();
                                        GCHandle pinnedPacket = GCHandle.Alloc(data, GCHandleType.Pinned);
                                        networkInfo = (WinData.FileNetworkInformation)Marshal.PtrToStructure(
                                            pinnedPacket.AddrOfPinnedObject(), typeof(WinData.FileNetworkInformation));
                                        pinnedPacket.Free();

                                        if (dataStatus == DataBufferStatus.DisplayData)
                                        {

                                            AddItem("CreationTime", DateTime.FromFileTime(networkInfo.CreationTime).ToString("dd-MM-yyyy HH:mm:ss"));
                                            AddItem("LastAccessTime", DateTime.FromFileTime(networkInfo.LastAccessTime).ToString("dd-MM-yyyy HH:mm:ss"));
                                            AddItem("LastWriteTime", DateTime.FromFileTime(networkInfo.LastWriteTime).ToString("dd-MM-yyyy HH:mm:ss"));
                                            AddItem("ChangeTime", DateTime.FromFileTime(networkInfo.ChangeTime).ToString("dd-MM-yyyy HH:mm:ss"));
                                            AddItem("FileSize", networkInfo.FileSize.ToString());
                                            AddItem("AllocationSize", networkInfo.AllocationSize.ToString());
                                            AddItem("FileAttributes", ((FileAttributes)networkInfo.FileAttributes).ToString());
                                        }
                                        else
                                        {
                                            string CreationTime = listView_ReturnData.Items[0].SubItems[2].Text;
                                            string LastAccessTime = listView_ReturnData.Items[1].SubItems[2].Text;
                                            string LastWriteTime = listView_ReturnData.Items[2].SubItems[2].Text;
                                            string ChangeTime = listView_ReturnData.Items[3].SubItems[2].Text;
                                            string FileSize = listView_ReturnData.Items[4].SubItems[2].Text;
                                            string AllocationSize = listView_ReturnData.Items[5].SubItems[2].Text;
                                            string FileAttributes = listView_ReturnData.Items[6].SubItems[2].Text;

                                            networkInfo.CreationTime = DateTime.ParseExact(CreationTime, "dd-MM-yyyy HH:mm:ss", System.Globalization.CultureInfo.CurrentCulture).ToFileTime();
                                            networkInfo.LastAccessTime = DateTime.ParseExact(LastAccessTime, "dd-MM-yyyy HH:mm:ss", System.Globalization.CultureInfo.CurrentCulture).ToFileTime();
                                            networkInfo.LastWriteTime = DateTime.ParseExact(LastWriteTime, "dd-MM-yyyy HH:mm:ss", System.Globalization.CultureInfo.CurrentCulture).ToFileTime();
                                            networkInfo.ChangeTime = DateTime.ParseExact(ChangeTime, "dd-MM-yyyy HH:mm:ss", System.Globalization.CultureInfo.CurrentCulture).ToFileTime();
                                            networkInfo.FileSize = long.Parse(FileSize);
                                            networkInfo.AllocationSize = long.Parse(AllocationSize);
                                            networkInfo.FileAttributes = uint.Parse(FileAttributes);

                                            uint returnLength = (uint)Marshal.SizeOf(networkInfo);
                                            IntPtr returnData = Marshal.AllocHGlobal((int)returnLength);
                                            Marshal.StructureToPtr(networkInfo, returnData, true);
                                            Marshal.Copy(returnData, messageReply.DataBuffer, 0, (int)returnLength);
                                            messageReply.DataBufferLength = returnLength;
                                            Marshal.FreeHGlobal(returnData);
                                        }

                                        break;
                                    }

                                case WinData.FileInfomationClass.FileInternalInformation:
                                    {
                                        long fileId = BitConverter.ToInt64(data, 0);

                                        if (dataStatus == DataBufferStatus.DisplayData)
                                        {
                                            AddItem("FileId", fileId.ToString());
                                        }
                                        else
                                        {
                                            fileId = long.Parse(listView_ReturnData.Items[0].SubItems[2].Text);
                                            byte[] returnData = BitConverter.GetBytes(fileId);

                                            messageReply.DataBufferLength = (uint)returnData.Length;
                                            Array.Copy(returnData, messageReply.DataBuffer, returnData.Length);
                                        }

                                        break;
                                    }
                                default:
                                    {
                                        ret = false;
                                        break;
                                    }
                            }

                            break;
                        }
                }
            }
            catch (Exception ex)
            {
                EventMessage.LogErrorMessage("ProcessDataBuffer failed." + ex.Message);
                ret = false;
            }

            return ret;
        }


        private void listView_ReturnData_DoubleClick(object sender, EventArgs e)
        {
            string itemName = listView_ReturnData.SelectedItems[0].SubItems[1].Text;
            string value = listView_ReturnData.SelectedItems[0].SubItems[2].Text;
            if (DialogResult.OK == Input.InputBox(itemName, "Input new value:", ref value))
            {
                listView_ReturnData.SelectedItems[0].SubItems[2].Text = value;
            }
        }


        private void button_FilterStatus_Click(object sender, EventArgs e)
        {
            OptionForm optionForm = new OptionForm(OptionForm.OptionType.Filter_Status, textBox_FilterStatus.Text);
            if (DialogResult.OK == optionForm.ShowDialog())
            {
                textBox_FilterStatus.Text = optionForm.FilterStatus.ToString();
            }
            optionForm.Close();
        }

        private void button_ReturnStatus_Click(object sender, EventArgs e)
        {
            OptionForm optionForm = new OptionForm(OptionForm.OptionType.Return_Status, textBox_ReturnStatus.Text);
            if (DialogResult.OK == optionForm.ShowDialog())
            {
                textBox_ReturnStatus.Text = optionForm.ReturnStatus.ToString();
            }
            optionForm.Close();
        }

        private void button_Ok_Click(object sender, EventArgs e)
        {
            try
            {
                uint filterStatus = uint.Parse(textBox_FilterStatus.Text);
                messageReply.FilterStatus = filterStatus;
                messageReply.ReturnStatus = uint.Parse(textBox_ReturnStatus.Text);
            
                if (((uint)EaseFilter.FilterStatus.FILTER_DATA_BUFFER_IS_UPDATED & filterStatus) != 0)
                {
                    ProcessDataBuffer(DataBufferStatus.ReturnData);

                    return;
                }

            }
            catch (Exception ex)
            {
                EventMessage.LogErrorMessage("Apply return data failed." + ex.Message);
            }

            return;
        }
    }
}
